/**
 * Stream monitoring and recovery for Icecast
 */

class StreamMonitor {
    // Configuration
    static MONITOR_INTERVAL = 10000; // 10 seconds
    static DEBUG = false; // Set to true to enable debug logging

    /**
     * Constructor - initialize the stream monitor
     * @param {string} audioElementId - The ID of the audio element to monitor
     */
    constructor({ audioElementId }) {
        this.audioElement = document.getElementById(audioElementId);
        
        // Track stream state to avoid repeated notifications
        this.streamWasFoundBefore = false;
        this.lastError = null;
        this.consecutiveErrors = 0;
        
        // Track stream availability for automatic refresh
        this.hasActiveStreams = false;
        this.refreshCheckInterval = null;
        
        // Check if we're in no-stream mode (only enable refresh checks when offline)
        this.isNoStreamMode = document.querySelector('.no-stream') !== null;
        
        // Set up monitoring
        this.setupMonitoring();
    }
    
    /**
     * Debug logging function
     */
    debug(message, data) {
        if (StreamMonitor.DEBUG) {
            if (data) {
                console.log(`[StreamMonitor] ${message}`, data);
            } else {
                console.log(`[StreamMonitor] ${message}`);
            }
        }
    }
    
    /**
     * Set up monitoring interval
     */
    setupMonitoring() {
        // Only set up stream availability checking when in no-stream mode
        if (this.isNoStreamMode) {
            this.debug('Page is in no-stream mode. Enabling automatic refresh checks.');
            // Check for stream availability every 15 seconds
            this.refreshCheckInterval = setInterval(() => this.checkStreamAvailability(), 5000);
            // Initial availability check after 5 seconds
            setTimeout(() => this.checkStreamAvailability(), 2500);
        } else {
            this.debug('Stream is already available. Disabling automatic refresh checks.');
        }
        
        // Regular stream monitoring for the player if it exists
        if (this.audioElement) {
            // Check stream status periodically
            setInterval(() => this.updateStreamInfo(), StreamMonitor.MONITOR_INTERVAL);
            // Initial check
            setTimeout(() => this.updateStreamInfo(), 3000);
        }
    }
    
    /**
     * Function to update stream info and check status
     */
    updateStreamInfo() {
        // Skip if audio element doesn't exist or is paused
        if (!this.audioElement || this.audioElement.paused) {
            this.debug('Audio is unavailable or paused, skipping stream check');
            return;
        }
        
        this.debug('Audio is playing, checking stream status');
        
        fetch('status-json.xsl')
            .then(response => {
                if (!response.ok) {
                    throw new Error('HTTP error, status: ' + response.status);
                }
                return response.text();
            })
            .then(text => {
                try {
                    const data = JSON.parse(text);
                    
                    // Handle the format: {icestats: {source: {...}}}
                    if (data.icestats && data.icestats.source) {
                        this.debug('Found source data', data.icestats.source);
                        this.updateUIFromSource(data.icestats.source);
                        this.handleStreamFound();
                    } else {
                        this.debug('Source data not found in expected format', data);
                        this.handleStreamNotFound();
                    }
                } catch (error) {
                    console.error('Error parsing JSON:', error);
                    this.handleError('json-parse-error');
                }
            })
            .catch(error => {
                console.error('Error fetching stream data:', error);
                this.handleError('fetch-error');
            });
    }
    
    /**
     * Handle when stream is found
     */
    handleStreamFound() {
        // Reset error counter
        this.consecutiveErrors = 0;
        this.streamWasFoundBefore = true;
        
        // If audio element doesn't exist, nothing to do
        if (!this.audioElement) return;
        
        // If audio was troubled before, notify recovery
        if (this.lastError) {
            this.debug('Stream recovered after previous error');
            this.lastError = null;
            
            // Dispatch recovery event
            this.audioElement.dispatchEvent(new CustomEvent('stream-monitor:playback-started', {
                detail: { reason: 'stream-recovered' }
            }));
        }
    }
    
    /**
     * Handle when stream is not found
     */
    handleStreamNotFound() {
        // Skip if audio element doesn't exist
        if (!this.audioElement) return;
        
        // Only trigger problem if audio is playing and we've seen the stream before
        if (!this.audioElement.paused && this.audioElement.currentTime > 0 && this.streamWasFoundBefore) {
            this.consecutiveErrors++;
            
            // Only report after multiple consecutive errors
            if (this.consecutiveErrors >= 3) {
                console.warn('Stream source not found after multiple checks');
                this.audioElement.dispatchEvent(new CustomEvent('stream-monitor:problem-detected', {
                    detail: { reason: 'source-not-found' }
                }));
            }
        } else {
            // Audio is playing despite source not found
            this.debug('Audio is playing despite source not found in stats - ignoring');
        }
    }
    
    /**
     * Handle errors in fetching or parsing stream data
     */
    handleError(errorType) {
        // Skip if audio element doesn't exist
        if (!this.audioElement) return;
        
        // Only dispatch error if audio should be playing
        if (!this.audioElement.paused && this.audioElement.currentTime > 0) {
            this.consecutiveErrors++;
            
            if (this.consecutiveErrors >= 2) {
                this.lastError = errorType;
                this.audioElement.dispatchEvent(new CustomEvent('stream-monitor:problem-detected', {
                    detail: { reason: errorType }
                }));
            }
        }
    }
    
    /**
     * Update UI elements from source data
     */
    updateUIFromSource(source) {
        // Update title if available
        if (source.title && document.getElementById('stream-title')) {
            document.getElementById('stream-title').textContent = source.title;
        }
        
        // Update listener count
        if (source.listeners !== undefined && document.getElementById('stream-listeners')) {
            document.getElementById('stream-listeners').textContent = source.listeners + ' listeners';
        }
    }
    
    /**
     * Check if streams are available and refresh page when they appear
     */
    checkStreamAvailability() {
        this.debug('Checking for stream availability');
        
        fetch('status-json.xsl')
            .then(response => {
                if (!response.ok) {
                    throw new Error('HTTP error, status: ' + response.status);
                }
                return response.text();
            })
            .then(text => {
                try {
                    const data = JSON.parse(text);
                    const hasStreams = this.countAvailableStreams(data) > 0;
                    
                    // If we previously had no streams and now we do, refresh the page
                    if (!this.hasActiveStreams && hasStreams) {
                        this.debug('Streams are now available! Refreshing page...');
                        
                        // Clean up interval before refreshing
                        if (this.refreshCheckInterval) {
                            clearInterval(this.refreshCheckInterval);
                            this.refreshCheckInterval = null;
                        }
                        
                        // Add a small delay before refreshing
                        setTimeout(() => window.location.reload(), 1000);
                    }
                    
                    // Update our state
                    this.hasActiveStreams = hasStreams;
                    
                } catch (error) {
                    console.error('Error parsing JSON:', error);
                }
            })
            .catch(error => {
                console.error('Error checking stream availability:', error);
            });
    }
    
    /**
     * Count the number of available streams from Icecast data
     */
    countAvailableStreams(data) {
        if (!data.icestats) return 0;
        
        // Handle both single source and array of sources
        if (data.icestats.source) {
            if (Array.isArray(data.icestats.source)) {
                return data.icestats.source.length;
            } else {
                return 1; // Single source object
            }
        }
        
        return 0;
    }
}
